@@ -113,12 +113,6 @@ class flexible_table {
      */
     var $downloadable = false;
 
-    /**
-     * @var string which download plugin to use. Default '' means none - print
-     * html table with paging.
-     */
-    var $defaultdownloadformat  = 'csv';
-
     /**
      * @var bool Has start output been called yet?
      */
@@ -133,7 +127,7 @@ class flexible_table {
 
     /**
      * Constructor
-     * @param int $uniqueid all tables have to have a unique id, this is used
+     * @param string $uniqueid all tables have to have a unique id, this is used
      *      as a key when storing table properties like sort order in the session.
      */
     function __construct($uniqueid) {
@@ -157,10 +151,10 @@ class flexible_table {
      * for you (even if the param is '', which means no download this time.
      * Also you can call this method with no params to get the current set
      * download type.
-     * @param string $download download type. One of csv, tsv, xhtml, ods, etc
+     * @param string $download dataformat type. One of csv, xhtml, ods, etc
      * @param string $filename filename for downloads without file extension.
      * @param string $sheettitle title for downloaded data.
-     * @return string download type.  One of csv, tsv, xhtml, ods, etc
+     * @return string download dataformat type. One of csv, xhtml, ods, etc
      */
     function is_downloading($download = null, $filename='', $sheettitle='') {
         if ($download!==null) {
@@ -184,8 +178,7 @@ class flexible_table {
             $this->exportclass = $exportclass;
             $this->exportclass->table = $this;
         } else if (is_null($this->exportclass) && !empty($this->download)) {
-            $classname = 'table_'.$this->download.'_export_format';
-            $this->exportclass = new $classname($this);
+            $this->exportclass = new table_dataformat_export_format($this, $this->download);
             if (!$this->exportclass->document_started()) {
                 $this->exportclass->start_document($this->filename);
             }
@@ -1018,44 +1011,23 @@ class flexible_table {
         }
         return $row;
     }
-    /**
-     * This function is not part of the public api.
-     */
-    function get_download_menu() {
-        $allclasses= get_declared_classes();
-        $exportclasses = array();
-        foreach ($allclasses as $class) {
-            $matches = array();
-            if (preg_match('/^table\_([a-z]+)\_export\_format$/', $class, $matches)) {
-                $type = $matches[1];
-                $exportclasses[$type]= get_string("download$type", 'table');
-            }
-        }
-        return $exportclasses;
-    }
 
     /**
-     * This function is not part of the public api.
+     * Get the html for the download buttons
+     *
+     * Usually only use internally
      */
-    function download_buttons() {
+    public function download_buttons() {
+        global $OUTPUT;
+
         if ($this->is_downloadable() && !$this->is_downloading()) {
-            $downloadoptions = $this->get_download_menu();
-
-            $downloadelements = new stdClass();
-            $downloadelements->formatsmenu = html_writer::select($downloadoptions,
-                    'download', $this->defaultdownloadformat, false);
-            $downloadelements->downloadbutton = '<input type="submit" value="'.
-                    get_string('download').'"/>';
-            $html = '<form action="'. $this->baseurl .'" method="post">';
-            $html .= '<div class="mdl-align">';
-            $html .= html_writer::tag('label', get_string('downloadas', 'table', $downloadelements));
-            $html .= '</div></form>';
-
-            return $html;
+            return $OUTPUT->download_dataformat_selector(get_string('downloadas', 'table'),
+                    $this->baseurl->out_omit_querystring(), 'download', $this->baseurl->params());
         } else {
             return '';
         }
     }
+
     /**
      * This function is not part of the public api.
      * You don't normally need to call this. It is called automatically when
@@ -1669,9 +1641,12 @@ class table_default_export_format_parent {
     }
 
     /**
-     * Old syntax of class constructor for backward compatibility.
+     * Old syntax of class constructor. Deprecated in PHP7.
+     *
+     * @deprecated since Moodle 3.1
      */
     public function table_default_export_format_parent(&$table) {
+        debugging('Use of class name as constructor is deprecated', DEBUG_DEVELOPER);
         self::__construct($table);
     }
 
@@ -1702,328 +1677,105 @@ class table_default_export_format_parent {
     }
 }
 
-
 /**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * Dataformat exporter
+ *
+ * @package    core
+ * @subpackage tablelib
+ * @copyright  2016 Brendan Heywood (brendan@catalyst-au.net)
+ * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class table_spreadsheet_export_format_parent extends table_default_export_format_parent {
-    var $currentrow;
-    var $workbook;
-    var $worksheet;
-    /**
-     * @var object format object - format for normal table cells
-     */
-    var $formatnormal;
-    /**
-     * @var object format object - format for header table cells
-     */
-    var $formatheaders;
+class table_dataformat_export_format extends table_default_export_format_parent {
+
+    /** @var $dataformat */
+    protected $dataformat;
+
+    /** @var $rownum */
+    protected $rownum = 0;
+
+    /** @var $columns */
+    protected $columns;
 
     /**
-     * should be overriden in child class.
+     * Constructor
+     *
+     * @param string $table An sql table
+     * @param string $dataformat type of dataformat for export
      */
-    var $fileextension;
+    public function __construct(&$table, $dataformat) {
+        parent::__construct($table);
 
-    /**
-     * This method will be overridden in the child class.
-     */
-    function define_workbook() {
-    }
-
-    function start_document($filename) {
-        $filename = $filename.'.'.$this->fileextension;
-        $this->define_workbook();
-        // format types
-        $this->formatnormal = $this->workbook->add_format();
-        $this->formatnormal->set_bold(0);
-        $this->formatheaders = $this->workbook->add_format();
-        $this->formatheaders->set_bold(1);
-        $this->formatheaders->set_align('center');
-        // Sending HTTP headers
-        $this->workbook->send($filename);
-        $this->documentstarted = true;
-    }
-
-    function start_table($sheettitle) {
-        $this->worksheet = $this->workbook->add_worksheet($sheettitle);
-        $this->currentrow=0;
-    }
-
-    function output_headers($headers) {
-        $colnum = 0;
-        foreach ($headers as $item) {
-            $this->worksheet->write($this->currentrow,$colnum,$item,$this->formatheaders);
-            $colnum++;
+        if (ob_get_length()) {
+            throw new coding_exception("Output can not be buffered before instantiating table_dataformat_export_format");
         }
-        $this->currentrow++;
-    }
 
-    function add_data($row) {
-        $colnum = 0;
-        foreach ($row as $item) {
-            $this->worksheet->write($this->currentrow,$colnum,$item,$this->formatnormal);
-            $colnum++;
+        $classname = 'dataformat_' . $dataformat . '\writer';
+        if (!class_exists($classname)) {
+            throw new coding_exception("Unable to locate dataformat/$dataformat/classes/writer.php");
         }
-        $this->currentrow++;
-        return true;
-    }
+        $this->dataformat = new $classname;
 
-    function add_seperator() {
-        $this->currentrow++;
-        return true;
-    }
+        // The dataformat export time to first byte could take a while to generate...
+        set_time_limit(0);
 
-    function finish_table() {
-    }
-
-    function finish_document() {
-        $this->workbook->close();
-        exit;
-    }
-}
-
-
-/**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class table_excel_export_format extends table_spreadsheet_export_format_parent {
-    var $fileextension = 'xls';
-
-    function define_workbook() {
-        global $CFG;
-        require_once("$CFG->libdir/excellib.class.php");
-        // Creating a workbook
-        $this->workbook = new MoodleExcelWorkbook("-");
-    }
-
-}
-
-
-/**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class table_ods_export_format extends table_spreadsheet_export_format_parent {
-    var $fileextension = 'ods';
-    function define_workbook() {
-        global $CFG;
-        require_once("$CFG->libdir/odslib.class.php");
-        // Creating a workbook
-        $this->workbook = new MoodleODSWorkbook("-");
-    }
-}
-
-
-/**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class table_text_export_format_parent extends table_default_export_format_parent {
-    protected $seperator = "tab";
-    protected $mimetype = 'text/tab-separated-values';
-    protected $ext = '.txt';
-    protected $myexporter;
-
-    public function __construct() {
-        $this->myexporter = new csv_export_writer($this->seperator, '"', $this->mimetype);
+        // Close the session so that the users other tabs in the same session are not blocked.
+        \core\session\manager::write_close();
     }
 
+    /**
+     * Start document
+     *
+     * @param string $filename
+     */
     public function start_document($filename) {
         $this->filename = $filename;
         $this->documentstarted = true;
-        $this->myexporter->set_filename($filename, $this->ext);
-    }
-
-    public function start_table($sheettitle) {
-        //nothing to do here
-    }
-
-    public function output_headers($headers) {
-        $this->myexporter->add_data($headers);
-    }
-
-    public function add_data($row) {
-        $this->myexporter->add_data($row);
-        return true;
-    }
-
-    public function finish_table() {
-        //nothing to do here
-    }
-
-    public function finish_document() {
-        $this->myexporter->download_file();
-        exit;
+        $this->dataformat->set_filename($filename);
     }
 
     /**
-     * Format a row of data.
-     * @param array $data
+     * Start export
+     *
+     * @param string $sheettitle optional spreadsheet worksheet title
      */
-    protected function format_row($data) {
-        $escapeddata = array();
-        foreach ($data as $value) {
-            $escapeddata[] = '"' . str_replace('"', '""', $value) . '"';
-        }
-        return implode($this->seperator, $escapeddata) . "\n";
-    }
-}
-
-
-/**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class table_tsv_export_format extends table_text_export_format_parent {
-    protected $seperator = "tab";
-    protected $mimetype = 'text/tab-separated-values';
-    protected $ext = '.txt';
-}
-
-require_once($CFG->libdir . '/csvlib.class.php');
-/**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class table_csv_export_format extends table_text_export_format_parent {
-    protected $seperator = "comma";
-    protected $mimetype = 'text/csv';
-    protected $ext = '.csv';
-}
-
-/**
- * @package   moodlecore
- * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-class table_xhtml_export_format extends table_default_export_format_parent {
-    function start_document($filename) {
-        header("Content-Type: application/download\n");
-        header("Content-Disposition: attachment; filename=\"$filename.html\"");
-        header("Expires: 0");
-        header("Cache-Control: must-revalidate,post-check=0,pre-check=0");
-        header("Pragma: public");
-        //html headers
-        echo <<<EOF
-<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE html
-  PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
-  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-
-<html xmlns="http://www.w3.org/1999/xhtml"
-  xml:lang="en" lang="en">
-<head>
-<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
-<style type="text/css">/*<![CDATA[*/
-
-.flexible th {
-white-space:normal;
-}
-th.header, td.header, div.header {
-border-color:#DDDDDD;
-background-color:lightGrey;
-}
-.flexible th {
-white-space:nowrap;
-}
-th {
-font-weight:bold;
-}
-
-.generaltable {
-border-style:solid;
-}
-.generalbox {
-border-style:solid;
-}
-body, table, td, th {
-font-family:Arial,Verdana,Helvetica,sans-serif;
-font-size:100%;
-}
-td {
-    border-style:solid;
-    border-width:1pt;
-}
-table {
-    border-collapse:collapse;
-    border-spacing:0pt;
-    width:80%;
-    margin:auto;
-}
-
-h1, h2 {
-    text-align:center;
-}
-.bold {
-font-weight:bold;
-}
-.mdl-align {
-    text-align:center;
-}
-/*]]>*/</style>
-<title>$filename</title>
-</head>
-<body>
-EOF;
-        $this->documentstarted = true;
+    public function start_table($sheettitle) {
+        $this->dataformat->set_sheettitle($sheettitle);
+        $this->dataformat->send_http_headers();
     }
 
-    function start_table($sheettitle) {
-        $this->table->sortable(false);
-        $this->table->collapsible(false);
-        echo "<h2>{$sheettitle}</h2>";
-        $this->table->start_html();
+    /**
+     * Output headers
+     *
+     * @param array $headers
+     */
+    public function output_headers($headers) {
+        $this->columns = $headers;
+        $this->dataformat->write_header($headers);
     }
 
-    function output_headers($headers) {
-        $this->table->print_headers();
-        echo html_writer::start_tag('tbody');
-    }
-
-    function add_data($row) {
-        $this->table->print_row($row);
+    /**
+     * Add a row of data
+     *
+     * @param array $row One record of data
+     */
+    public function add_data($row) {
+        $this->dataformat->write_record($row, $this->rownum++);
         return true;
     }
 
-    function add_seperator() {
-        $this->table->print_row(NULL);
-        return true;
+    /**
+     * Finish export
+     */
+    public function finish_table() {
+        $this->dataformat->write_footer($this->columns);
     }
 
-    function finish_table() {
-        $this->table->finish_html();
-    }
-
-    function finish_document() {
-        echo "</body>\n</html>";
+    /**
+     * Finish download
+     */
+    public function finish_document() {
         exit;
     }
 
-    function format_text($text, $format=FORMAT_MOODLE, $options=NULL, $courseid=NULL) {
-        if (is_null($options)) {
-            $options = new stdClass;
-        }
-        //some sensible defaults
-        if (!isset($options->para)) {
-            $options->para = false;
-        }
-        if (!isset($options->newlines)) {
-            $options->newlines = false;
-        }
-        if (!isset($options->smiley)) {
-            $options->smiley = false;
-        }
-        if (!isset($options->filter)) {
-            $options->filter = false;
-        }
-        return format_text($text, $format, $options);
-    }
 }
+
