@@ -47,9 +47,13 @@ class core_test_generator_testcase extends advanced_testcase {
         $this->assertInstanceOf('mod_quiz_generator', $generator);
     }
 
+    /**
+     * Test plugin generator, with no component directory.
+     *
+     * @expectedException        coding_exception
+     * @expectedExceptionMessage Component core_completion does not support generators yet. Missing tests/generator/lib.php.
+     */
     public function test_get_plugin_generator_no_component_dir() {
-        $this->setExpectedException('coding_exception', 'Component core_completion does not support ' .
-                    'generators yet. Missing tests/generator/lib.php.');
         $generator = $this->getDataGenerator()->get_plugin_generator('core_completion');
     }
 
