@@ -24,9 +24,8 @@
  */
 
 defined('MOODLE_INTERNAL') || die();
-/**
- * File areas for file feedback assignment
- */
+
+// File areas for file feedback assignment.
 define('ASSIGNFEEDBACK_FILE_FILEAREA', 'feedback_files');
 define('ASSIGNFEEDBACK_FILE_BATCH_FILEAREA', 'feedback_files_batch');
 define('ASSIGNFEEDBACK_FILE_IMPORT_FILEAREA', 'feedback_files_import');
@@ -35,16 +34,17 @@ define('ASSIGNFEEDBACK_FILE_MAXSUMMARYUSERS', 5);
 define('ASSIGNFEEDBACK_FILE_MAXFILEUNZIPTIME', 120);
 
 /**
- * library class for file feedback plugin extending feedback plugin base class
+ * Library class for file feedback plugin extending feedback plugin base class.
  *
- * @package   asignfeedback_file
+ * @package   assignfeedback_file
  * @copyright 2012 NetSpot {@link http://www.netspot.com.au}
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class assign_feedback_file extends assign_feedback_plugin {
 
     /**
-     * Get the name of the file feedback plugin
+     * Get the name of the file feedback plugin.
+     *
      * @return string
      */
     public function get_name() {
@@ -52,7 +52,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Get file feedback information from the database
+     * Get file feedback information from the database.
      *
      * @param int $gradeid
      * @return mixed
@@ -63,21 +63,86 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * File format options
+     * File format options.
+     *
      * @return array
      */
     private function get_file_options() {
         global $COURSE;
 
         $fileoptions = array('subdirs'=>1,
-                                'maxbytes'=>$COURSE->maxbytes,
-                                'accepted_types'=>'*',
-                                'return_types'=>FILE_INTERNAL);
+                             'maxbytes'=>$COURSE->maxbytes,
+                             'accepted_types'=>'*',
+                             'return_types'=>FILE_INTERNAL);
         return $fileoptions;
     }
 
     /**
-     * Copy all the files from one file area to another
+     * Has the feedback file been modified?
+     *
+     * @param stdClass $grade Grade object.
+     * @param stdClass $data Form data.
+     * @return boolean True if the file area has been modified, else false.
+     */
+    public function is_feedback_modified(stdClass $grade, stdClass $data) {
+        global $USER;
+
+        $filekey = null;
+        $draftareainfo = null;
+        foreach ($data as $key => $value) {
+            if (strpos($key, 'files_') === 0 && strpos($key, '_filemanager')) {
+                $filekey = $key;
+            }
+        }
+        if (isset($filekey)) {
+            $draftareainfo = file_get_draft_area_info($data->$filekey);
+            $filecount = $this->count_files($grade->id, ASSIGNFEEDBACK_FILE_FILEAREA);
+            if ($filecount != $draftareainfo['filecount']) {
+                return true;
+            } else {
+                // We need to check that the files in the draft area are the same as in the file area.
+                $usercontext = context_user::instance($USER->id);
+                $fs = get_file_storage();
+                $draftfiles = $fs->get_area_files($usercontext->id, 'user', 'draft', $data->$filekey, 'id', true);
+                $files = $fs->get_area_files($this->assignment->get_context()->id,
+                                     'assignfeedback_file',
+                                     ASSIGNFEEDBACK_FILE_FILEAREA,
+                                     $grade->id,
+                                     'id',
+                                     false);
+                foreach ($files as $key => $file) {
+                    // Flag for recording if we have a matching file.
+                    $matchflag = false;
+                    foreach ($draftfiles as $draftkey => $draftfile) {
+                        if (!$file->is_directory()) {
+                            // File name is the same, but it could be a different file with the same name.
+                            if ($draftfile->get_filename() == $file->get_filename()) {
+                                // If the file name is the same but the content hash is different, or
+                                // The file path for the file has changed, then we have a modification.
+                                if ($draftfile->get_contenthash() != $file->get_contenthash() ||
+                                        $draftfile->get_filepath() != $file->get_filepath()) {
+                                    return true;
+                                }
+                                // These files match. Check the next file.
+                                $matchflag = true;
+                                // We have a match on the file name so we can move to the next file and not
+                                // proceed through the other draftfiles.
+                                break;
+                            }
+                        }
+                    }
+                    // If the file does not match then there has been a modification.
+                    if (!$matchflag) {
+                        return true;
+                    }
+                }
+            }
+        }
+        return false;
+    }
+
+    /**
+     * Copy all the files from one file area to another.
      *
      * @param file_storage $fs - The source context id
      * @param int $fromcontextid - The source context id
@@ -110,7 +175,8 @@ class assign_feedback_file extends assign_feedback_plugin {
             foreach ($files as $file) {
                 if ($file->is_directory() and $file->get_filepath() === '/') {
                     // We need a way to mark the age of each draft area.
-                    // By not copying the root dir we force it to be created automatically with current timestamp.
+                    // By not copying the root dir we force it to be created
+                    // automatically with current timestamp.
                     continue;
                 }
                 $newfile = $fs->create_file_from_storedfile($newfilerecord, $file);
@@ -120,7 +186,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Get form elements for grading form
+     * Get form elements for grading form.
      *
      * @param stdClass $grade
      * @param MoodleQuickForm $mform
@@ -141,14 +207,13 @@ class assign_feedback_file extends assign_feedback_plugin {
                                                   'assignfeedback_file',
                                                   ASSIGNFEEDBACK_FILE_FILEAREA,
                                                   $gradeid);
-
-        $mform->addElement('filemanager', $elementname . '_filemanager', '', null, $fileoptions);
+        $mform->addElement('filemanager', $elementname . '_filemanager', $this->get_name(), null, $fileoptions);
 
         return true;
     }
 
     /**
-     * Count the number of files
+     * Count the number of files.
      *
      * @param int $gradeid
      * @param string $area
@@ -157,13 +222,18 @@ class assign_feedback_file extends assign_feedback_plugin {
     private function count_files($gradeid, $area) {
 
         $fs = get_file_storage();
-        $files = $fs->get_area_files($this->assignment->get_context()->id, 'assignfeedback_file', $area, $gradeid, "id", false);
+        $files = $fs->get_area_files($this->assignment->get_context()->id,
+                                     'assignfeedback_file',
+                                     $area,
+                                     $gradeid,
+                                     'id',
+                                     false);
 
         return count($files);
     }
 
     /**
-     * Update the number of files in the file area
+     * Update the number of files in the file area.
      *
      * @param stdClass $grade The grade record
      * @return bool - true if the value was saved
@@ -185,7 +255,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Save the feedback files
+     * Save the feedback files.
      *
      * @param stdClass $grade
      * @param stdClass $data
@@ -213,48 +283,57 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Display the list of files  in the feedback status table
+     * Display the list of files in the feedback status table.
      *
      * @param stdClass $grade
      * @param bool $showviewlink - Set to true to show a link to see the full list of files
      * @return string
      */
     public function view_summary(stdClass $grade, & $showviewlink) {
+
         $count = $this->count_files($grade->id, ASSIGNFEEDBACK_FILE_FILEAREA);
-        // show a view all link if the number of files is over this limit
+
+        // Show a view all link if the number of files is over this limit.
         $showviewlink = $count > ASSIGNFEEDBACK_FILE_MAXSUMMARYFILES;
 
         if ($count <= ASSIGNFEEDBACK_FILE_MAXSUMMARYFILES) {
-            return $this->assignment->render_area_files('assignfeedback_file', ASSIGNFEEDBACK_FILE_FILEAREA, $grade->id);
+            return $this->assignment->render_area_files('assignfeedback_file',
+                                                        ASSIGNFEEDBACK_FILE_FILEAREA,
+                                                        $grade->id);
         } else {
             return get_string('countfiles', 'assignfeedback_file', $count);
         }
     }
 
     /**
-     * Display the list of files  in the feedback status table
+     * Display the list of files in the feedback status table.
+     *
      * @param stdClass $grade
      * @return string
      */
     public function view(stdClass $grade) {
-        return $this->assignment->render_area_files('assignfeedback_file', ASSIGNFEEDBACK_FILE_FILEAREA, $grade->id);
+        return $this->assignment->render_area_files('assignfeedback_file',
+                                                    ASSIGNFEEDBACK_FILE_FILEAREA,
+                                                    $grade->id);
     }
 
     /**
-     * The assignment has been deleted - cleanup
+     * The assignment has been deleted - cleanup.
      *
      * @return bool
      */
     public function delete_instance() {
         global $DB;
-        // will throw exception on failure
-        $DB->delete_records('assignfeedback_file', array('assignment'=>$this->assignment->get_instance()->id));
+        // Will throw exception on failure.
+        $DB->delete_records('assignfeedback_file',
+                            array('assignment'=>$this->assignment->get_instance()->id));
 
         return true;
     }
 
     /**
-     * Return true if there are no feedback files
+     * Return true if there are no feedback files.
+     *
      * @param stdClass $grade
      */
     public function is_empty(stdClass $grade) {
@@ -262,7 +341,8 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Get file areas returns a list of areas this plugin stores files
+     * Get file areas returns a list of areas this plugin stores files.
+     *
      * @return array - An array of fileareas (keys) and descriptions (values)
      */
     public function get_file_areas() {
@@ -278,7 +358,6 @@ class assign_feedback_file extends assign_feedback_plugin {
      * @return bool True if upgrade is possible
      */
     public function can_upgrade($type, $version) {
-
         if (($type == 'upload' || $type == 'uploadsingle') && $version >= 2011112900) {
             return true;
         }
@@ -286,7 +365,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Upgrade the settings from the old assignment to the new plugin based one
+     * Upgrade the settings from the old assignment to the new plugin based one.
      *
      * @param context $oldcontext - the context for the old assignment
      * @param stdClass $oldassignment - the data for the old assignment
@@ -294,12 +373,12 @@ class assign_feedback_file extends assign_feedback_plugin {
      * @return bool was it a success? (false will trigger a rollback)
      */
     public function upgrade_settings(context $oldcontext, stdClass $oldassignment, & $log) {
-        // first upgrade settings (nothing to do)
+        // First upgrade settings (nothing to do).
         return true;
     }
 
     /**
-     * Upgrade the feedback from the old assignment to the new one
+     * Upgrade the feedback from the old assignment to the new one.
      *
      * @param context $oldcontext - the database for the old assignment context
      * @param stdClass $oldassignment The data record for the old assignment
@@ -308,21 +387,24 @@ class assign_feedback_file extends assign_feedback_plugin {
      * @param string $log Record upgrade messages in the log
      * @return bool true or false - false will trigger a rollback
      */
-    public function upgrade(context $oldcontext, stdClass $oldassignment, stdClass $oldsubmission, stdClass $grade, & $log) {
+    public function upgrade(context $oldcontext,
+                            stdClass $oldassignment,
+                            stdClass $oldsubmission,
+                            stdClass $grade,
+                            & $log) {
         global $DB;
 
-        // now copy the area files
+        // Now copy the area files.
         $this->assignment->copy_area_files_for_upgrade($oldcontext->id,
                                                         'mod_assignment',
                                                         'response',
                                                         $oldsubmission->id,
-                                                        // New file area
                                                         $this->assignment->get_context()->id,
                                                         'assignfeedback_file',
                                                         ASSIGNFEEDBACK_FILE_FILEAREA,
                                                         $grade->id);
 
-        // now count them!
+        // Now count them!
         $filefeedback = new stdClass();
         $filefeedback->numfiles = $this->count_files($grade->id, ASSIGNFEEDBACK_FILE_FILEAREA);
         $filefeedback->grade = $grade->id;
@@ -335,8 +417,8 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Return a list of the batch grading operations performed by this plugin
-     * This plugin supports batch upload files and upload zip
+     * Return a list of the batch grading operations performed by this plugin.
+     * This plugin supports batch upload files and upload zip.
      *
      * @return array The list of batch grading operations
      */
@@ -345,7 +427,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Upload files and send them to multiple users
+     * Upload files and send them to multiple users.
      *
      * @param array $users - An array of user ids
      * @return string - The response html
@@ -366,17 +448,20 @@ class assign_feedback_file extends assign_feedback_plugin {
         $usercount = 0;
         foreach ($users as $userid) {
             if ($usercount >= ASSIGNFEEDBACK_FILE_MAXSUMMARYUSERS) {
-                $usershtml .= get_string('moreusers', 'assignfeedback_file', count($users) - ASSIGNFEEDBACK_FILE_MAXSUMMARYUSERS);
+                $moreuserscount = count($users) - ASSIGNFEEDBACK_FILE_MAXSUMMARYUSERS;
+                $usershtml .= get_string('moreusers', 'assignfeedback_file', $moreuserscount);
                 break;
             }
             $user = $DB->get_record('user', array('id'=>$userid), '*', MUST_EXIST);
 
-            $usershtml .= $this->assignment->get_renderer()->render(new assign_user_summary($user,
-                                                                $this->assignment->get_course()->id,
-                                                                has_capability('moodle/site:viewfullnames',
-                                                                $this->assignment->get_course_context()),
-                                                                $this->assignment->is_blind_marking(),
-                                                                $this->assignment->get_uniqueid_for_user($user->id)));
+            $usersummary = new assign_user_summary($user,
+                                                   $this->assignment->get_course()->id,
+                                                   has_capability('moodle/site:viewfullnames',
+                                                   $this->assignment->get_course_context()),
+                                                   $this->assignment->is_blind_marking(),
+                                                   $this->assignment->get_uniqueid_for_user($user->id),
+                                                   get_extra_user_fields($this->assignment->get_context()));
+            $usershtml .= $this->assignment->get_renderer()->render($usersummary);
             $usercount += 1;
         }
 
@@ -403,6 +488,7 @@ class assign_feedback_file extends assign_feedback_plugin {
             // Now copy each of these files to the users feedback file area.
             foreach ($users as $userid) {
                 $grade = $this->assignment->get_user_grade($userid, true);
+                $this->assignment->notify_grade_modified($grade);
 
                 $this->copy_area_files($fs,
                                        $this->assignment->get_context()->id,
@@ -416,11 +502,13 @@ class assign_feedback_file extends assign_feedback_plugin {
 
                 $filefeedback = $this->get_file_feedback($grade->id);
                 if ($filefeedback) {
-                    $filefeedback->numfiles = $this->count_files($grade->id, ASSIGNFEEDBACK_FILE_FILEAREA);
+                    $filefeedback->numfiles = $this->count_files($grade->id,
+                                                                 ASSIGNFEEDBACK_FILE_FILEAREA);
                     $DB->update_record('assignfeedback_file', $filefeedback);
                 } else {
                     $filefeedback = new stdClass();
-                    $filefeedback->numfiles = $this->count_files($grade->id, ASSIGNFEEDBACK_FILE_FILEAREA);
+                    $filefeedback->numfiles = $this->count_files($grade->id,
+                                                                 ASSIGNFEEDBACK_FILE_FILEAREA);
                     $filefeedback->grade = $grade->id;
                     $filefeedback->assignment = $this->assignment->get_instance()->id;
                     $DB->insert_record('assignfeedback_file', $filefeedback);
@@ -439,12 +527,13 @@ class assign_feedback_file extends assign_feedback_plugin {
             return;
         } else {
 
+            $header = new assign_header($this->assignment->get_instance(),
+                                        $this->assignment->get_context(),
+                                        false,
+                                        $this->assignment->get_course_module()->id,
+                                        get_string('batchuploadfiles', 'assignfeedback_file'));
             $o = '';
-            $o .= $this->assignment->get_renderer()->render(new assign_header($this->assignment->get_instance(),
-                                                          $this->assignment->get_context(),
-                                                          false,
-                                                          $this->assignment->get_course_module()->id,
-                                                          get_string('batchuploadfiles', 'assignfeedback_file')));
+            $o .= $this->assignment->get_renderer()->render($header);
             $o .= $this->assignment->get_renderer()->render(new assign_form('batchuploadfiles', $mform));
             $o .= $this->assignment->get_renderer()->render_footer();
         }
@@ -453,7 +542,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * User has chosen a custom grading batch operation and selected some users
+     * User has chosen a custom grading batch operation and selected some users.
      *
      * @param string $action - The chosen action
      * @param array $users - An array of user ids
@@ -468,7 +557,7 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * View the upload zip form
+     * View the upload zip form.
      *
      * @return string - The html response
      */
@@ -480,23 +569,25 @@ class assign_feedback_file extends assign_feedback_plugin {
         require_once($CFG->dirroot . '/mod/assign/feedback/file/importziplib.php');
         require_once($CFG->dirroot . '/mod/assign/feedback/file/importzipform.php');
 
-        $mform = new assignfeedback_file_upload_zip_form(null,
-                                                          array('context'=>$this->assignment->get_context(),
-                                                                'cm'=>$this->assignment->get_course_module()->id));
+        $formparams = array('context'=>$this->assignment->get_context(),
+                            'cm'=>$this->assignment->get_course_module()->id);
+        $mform = new assignfeedback_file_upload_zip_form(null, $formparams);
 
         $o = '';
 
         $confirm = optional_param('confirm', 0, PARAM_BOOL);
         $renderer = $this->assignment->get_renderer();
+
         // Delete any existing files.
         $importer = new assignfeedback_file_zip_importer();
         $contextid = $this->assignment->get_context()->id;
 
         if ($mform->is_cancelled()) {
             $importer->delete_import_files($contextid);
-            redirect(new moodle_url('view.php',
-                                    array('id'=>$this->assignment->get_course_module()->id,
-                                          'action'=>'grading')));
+            $urlparams = array('id'=>$this->assignment->get_course_module()->id,
+                               'action'=>'grading');
+            $url = new moodle_url('view.php', $urlparams);
+            redirect($url);
             return;
         } else if ($confirm) {
             $params = array('assignment'=>$this->assignment, 'importer'=>$importer);
@@ -504,9 +595,10 @@ class assign_feedback_file extends assign_feedback_plugin {
             $mform = new assignfeedback_file_import_zip_form(null, $params);
             if ($mform->is_cancelled()) {
                 $importer->delete_import_files($contextid);
-                redirect(new moodle_url('view.php',
-                                        array('id'=>$this->assignment->get_course_module()->id,
-                                          'action'=>'grading')));
+                $urlparams = array('id'=>$this->assignment->get_course_module()->id,
+                                   'action'=>'grading');
+                $url = new moodle_url('view.php', $urlparams);
+                redirect($url);
                 return;
             }
 
@@ -528,21 +620,23 @@ class assign_feedback_file extends assign_feedback_plugin {
 
             $mform = new assignfeedback_file_import_zip_form(null, $params);
 
-            $o .= $renderer->render(new assign_header($this->assignment->get_instance(),
-                                                            $this->assignment->get_context(),
-                                                            false,
-                                                            $this->assignment->get_course_module()->id,
-                                                            get_string('confirmuploadzip', 'assignfeedback_file')));
+            $header = new assign_header($this->assignment->get_instance(),
+                                        $this->assignment->get_context(),
+                                        false,
+                                        $this->assignment->get_course_module()->id,
+                                        get_string('confirmuploadzip', 'assignfeedback_file'));
+            $o .= $renderer->render($header);
             $o .= $renderer->render(new assign_form('confirmimportzip', $mform));
             $o .= $renderer->render_footer();
 
         } else {
 
-            $o .= $renderer->render(new assign_header($this->assignment->get_instance(),
-                                                            $this->assignment->get_context(),
-                                                            false,
-                                                            $this->assignment->get_course_module()->id,
-                                                            get_string('uploadzip', 'assignfeedback_file')));
+            $header = new assign_header($this->assignment->get_instance(),
+                                        $this->assignment->get_context(),
+                                        false,
+                                        $this->assignment->get_course_module()->id,
+                                        get_string('uploadzip', 'assignfeedback_file'));
+            $o .= $renderer->render($header);
             $o .= $renderer->render(new assign_form('uploadfeedbackzip', $mform));
             $o .= $renderer->render_footer();
         }
@@ -551,14 +645,15 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Called by the assignment module when someone chooses something from the grading navigation or batch operations list
+     * Called by the assignment module when someone chooses something from the
+     * grading navigation or batch operations list.
      *
      * @param string $action - The page to view
      * @return string - The html response
      */
     public function view_page($action) {
         if ($action == 'uploadfiles') {
-            $users = required_param('selectedusers', PARAM_TEXT);
+            $users = required_param('selectedusers', PARAM_SEQUENCE);
             return $this->view_batch_upload_files(explode(',', $users));
         }
         if ($action == 'uploadzip') {
@@ -569,12 +664,37 @@ class assign_feedback_file extends assign_feedback_plugin {
     }
 
     /**
-     * Return a list of the grading actions performed by this plugin
-     * This plugin supports upload zip
+     * Return a list of the grading actions performed by this plugin.
+     * This plugin supports upload zip.
      *
      * @return array The list of grading actions
      */
     public function get_grading_actions() {
         return array('uploadzip'=>get_string('uploadzip', 'assignfeedback_file'));
     }
+
+    /**
+     * Return a description of external params suitable for uploading a feedback file from a webservice.
+     *
+     * @return external_description|null
+     */
+    public function get_external_parameters() {
+        return array(
+            'files_filemanager' => new external_value(
+                PARAM_INT,
+                'The id of a draft area containing files for this feedback.',
+                VALUE_OPTIONAL
+            )
+        );
+    }
+
+    /**
+     * Return the plugin configs for external functions.
+     *
+     * @return array the list of settings
+     * @since Moodle 3.2
+     */
+    public function get_config_for_external() {
+        return (array) $this->get_config();
+    }
 }
