@@ -17,15 +17,14 @@
 /**
  * scorm version information.
  *
- * @package    mod
- * @subpackage scorm
+ * @package    mod_scorm
  * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 defined('MOODLE_INTERNAL') || die();
 
-$module->version   = 2013110500;       // The current module version (Date: YYYYMMDDXX)
-$module->requires  = 2013110500;    // Requires this Moodle version
-$module->component = 'mod_scorm'; // Full name of the plugin (used for diagnostics)
-$module->cron      = 300;
+$plugin->version   = 2016120500;    // The current module version (Date: YYYYMMDDXX).
+$plugin->requires  = 2016112900;    // Requires this Moodle version.
+$plugin->component = 'mod_scorm';   // Full name of the plugin (used for diagnostics).
+$plugin->cron      = 300;
