@@ -27,6 +27,7 @@
 
 defined('MOODLE_INTERNAL') || die();
 
+require_once($CFG->libdir . '/questionlib.php');
 require_once($CFG->dirroot . '/question/type/numerical/question.php');
 
 
@@ -74,8 +75,8 @@ class qtype_numerical extends question_type {
                 array('questionid' => $question->id), 'id ASC');
 
         $this->get_numerical_units($question);
-        // get_numerical_options() need to know if there are units
-        // to set correctly default values
+        // Get_numerical_options() need to know if there are units
+        // to set correctly default values.
         $this->get_numerical_options($question);
 
         // If units are defined we strip off the default unit from the answer, if
@@ -104,7 +105,7 @@ class qtype_numerical extends question_type {
             $units = array();
         }
         foreach ($units as $key => $unit) {
-            $units[$key]->multiplier = clean_param($unit->multiplier, PARAM_NUMBER);
+            $units[$key]->multiplier = clean_param($unit->multiplier, PARAM_FLOAT);
         }
         $question->options->units = $units;
         return true;
@@ -152,7 +153,7 @@ class qtype_numerical extends question_type {
         global $DB;
         $context = $question->context;
 
-        // Get old versions of the objects
+        // Get old versions of the objects.
         $oldanswers = $DB->get_records('question_answers',
                 array('question' => $question->id), 'id ASC');
         $oldoptions = $DB->get_records('question_numerical',
@@ -166,7 +167,7 @@ class qtype_numerical extends question_type {
             $units = $result->units;
         }
 
-        // Insert all the new answers
+        // Insert all the new answers.
         foreach ($question->answer as $key => $answerdata) {
             // Check for, and ingore, completely blank answer from the form.
             if (trim($answerdata) == '' && $question->fraction[$key] == 0 &&
@@ -190,7 +191,7 @@ class qtype_numerical extends question_type {
                 $answer->answer = $this->apply_unit($answerdata, $units,
                         !empty($question->unitsleft));
                 if ($answer->answer === false) {
-                    $result->notice = get_string('invalidnumericanswer', 'quiz');
+                    $result->notice = get_string('invalidnumericanswer', 'qtype_numerical');
                 }
             }
             $answer->fraction = $question->fraction[$key];
@@ -199,7 +200,7 @@ class qtype_numerical extends question_type {
             $answer->feedbackformat = $question->feedback[$key]['format'];
             $DB->update_record('question_answers', $answer);
 
-            // Set up the options object
+            // Set up the options object.
             if (!$options = array_shift($oldoptions)) {
                 $options = new stdClass();
             }
@@ -211,7 +212,7 @@ class qtype_numerical extends question_type {
                 $options->tolerance = $this->apply_unit($question->tolerance[$key],
                         $units, !empty($question->unitsleft));
                 if ($options->tolerance === false) {
-                    $result->notice = get_string('invalidnumerictolerance', 'quiz');
+                    $result->notice = get_string('invalidnumerictolerance', 'qtype_numerical');
                 }
             }
             if (isset($options->id)) {
@@ -281,7 +282,9 @@ class qtype_numerical extends question_type {
         } else if (isset($question->showunits)) {
             // Updated import, e.g. Moodle XML.
             $options->showunits = $question->showunits;
-
+            if (isset($question->unitgradingtype)) {
+                $options->unitgradingtype = $question->unitgradingtype;
+            }
         } else {
             // Legacy import.
             if ($defaultunit = $this->get_default_numerical_unit($question)) {
@@ -320,7 +323,7 @@ class qtype_numerical extends question_type {
         $units = array();
         $unitalreadyinsert = array();
         foreach ($question->multiplier as $i => $multiplier) {
-            // Discard any unit which doesn't specify the unit or the multiplier
+            // Discard any unit which doesn't specify the unit or the multiplier.
             if (!empty($question->multiplier[$i]) && !empty($question->unit[$i]) &&
                     !array_key_exists($question->unit[$i], $unitalreadyinsert)) {
                 $unitalreadyinsert[$question->unit[$i]] = 1;
@@ -418,21 +421,30 @@ class qtype_numerical extends question_type {
 
         $unit = $this->get_default_numerical_unit($questiondata);
 
+        $starfound = false;
         foreach ($questiondata->options->answers as $aid => $answer) {
             $responseclass = $answer->answer;
 
-            if ($responseclass != '*') {
+            if ($responseclass === '*') {
+                $starfound = true;
+            } else {
                 $responseclass = $this->add_unit($questiondata, $responseclass, $unit);
 
                 $ans = new qtype_numerical_answer($answer->id, $answer->answer, $answer->fraction,
                         $answer->feedback, $answer->feedbackformat, $answer->tolerance);
                 list($min, $max) = $ans->get_tolerance_interval();
-                $responseclass .= " ($min..$max)";
+                $responseclass .= " ({$min}..{$max})";
             }
 
             $responses[$aid] = new question_possible_response($responseclass,
                     $answer->fraction);
         }
+
+        if (!$starfound) {
+            $responses[0] = new question_possible_response(
+                    get_string('didnotmatchanyanswer', 'question'), 0);
+        }
+
         $responses[null] = question_possible_response::no_response();
 
         return array($questiondata->id => $responses);
@@ -461,6 +473,7 @@ class qtype_numerical extends question_type {
 
         parent::move_files($questionid, $oldcontextid, $newcontextid);
         $this->move_files_in_answers($questionid, $oldcontextid, $newcontextid);
+        $this->move_files_in_hints($questionid, $oldcontextid, $newcontextid);
     }
 
     protected function delete_files($questionid, $contextid) {
@@ -468,6 +481,7 @@ class qtype_numerical extends question_type {
 
         parent::delete_files($questionid, $contextid);
         $this->delete_files_in_answers($questionid, $contextid);
+        $this->delete_files_in_hints($questionid, $contextid);
     }
 }
 
@@ -555,12 +569,12 @@ class qtype_numerical_answer_processor {
         $decimalsre = $decsep . '(\d*)';
         $exponentre = '(?:e|E|(?:x|\*|×)10(?:\^|\*\*))([+-]?\d+)';
 
-        $numberbit = "$beforepointre(?:$decimalsre)?(?:$exponentre)?";
+        $numberbit = "{$beforepointre}(?:{$decimalsre})?(?:{$exponentre})?";
 
         if ($this->unitsbefore) {
-            $this->regex = "/$numberbit$/";
+            $this->regex = "/{$numberbit}$/";
         } else {
-            $this->regex = "/^$numberbit/";
+            $this->regex = "/^{$numberbit}/";
         }
         return $this->regex;
     }
@@ -631,9 +645,9 @@ class qtype_numerical_answer_processor {
 
         $regex = '[+-]?(?:\d+(?:\\.\d*)?|\\.\d+)(?:e[-+]?\d+)?';
         if ($this->unitsbefore) {
-            $regex = "/$regex$/";
+            $regex = "/{$regex}$/";
         } else {
-            $regex = "/^$regex/";
+            $regex = "/^{$regex}/";
         }
         if (!preg_match($regex, $response, $matches)) {
             return array(null, null, null);
@@ -641,7 +655,7 @@ class qtype_numerical_answer_processor {
 
         $numberstring = $matches[0];
         if ($this->unitsbefore) {
-            // substr returns false when it means '', so cast back to string.
+            // Substr returns false when it means '', so cast back to string.
             $unit = (string) substr($response, 0, -strlen($numberstring));
         } else {
             $unit = (string) substr($response, strlen($numberstring));
@@ -657,7 +671,7 @@ class qtype_numerical_answer_processor {
             $multiplier = null;
         }
 
-        return array($numberstring + 0, $unit, $multiplier); // + 0 to convert to number.
+        return array($numberstring + 0, $unit, $multiplier); // The + 0 is to convert to number.
     }
 
     /**
